@@ -158,6 +158,12 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     hcolor = hGetColor(h);
     pkgKey = RPMAL_NOMATCH;
 
+    /* Check for supported payload format if it's a package */
+    if (key && headerCheckPayloadFormat(h) != RPMRC_OK) {
+	ec = 1;
+	goto exit;
+    }
+
     /* XXX Always add source headers. */
     isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
     if (isSource) {
@@ -195,9 +201,12 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	if ((this = rpmteDS(p, RPMTAG_NAME)) == NULL)
 	    continue;	/* XXX can't happen */
 
-	/* If newer NEVR was previously added, then skip adding older. */
+	/* 
+ 	 * On upgrade, if newer NEVR was previously added, 
+ 	 * then skip adding older. 
+ 	 */
 	rc = rpmdsCompare(newChk, this);
-	if (rc != 0) {
+	if (upgrade && rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
 	    const char * addNEVR = rpmdsDNEVR(oldChk);
 	    if (rpmIsVerbose())
@@ -209,9 +218,12 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	    break;
 	}
 
-	/* If older NEVR was previously added, then replace old with new. */
+	/*
+ 	 * On upgrade, if older NEVR was previously added, 
+ 	 * then replace old with new. 
+ 	 */
 	rc = rpmdsCompare(oldChk, this);
-	if (rc != 0) {
+	if (upgrade && rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
 	    const char * addNEVR = rpmdsDNEVR(newChk);
 	    if (rpmIsVerbose())
@@ -294,7 +306,7 @@ addheader:
 
     /* On upgrade, erase older packages of same color (if any). */
 
-    mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, rpmteN(p), 0);
+    mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(p), 0);
     while((oh = rpmdbNextIterator(mi)) != NULL) {
 
 	/* Ignore colored packages not in our rainbow. */
@@ -336,7 +348,7 @@ addheader:
 	if (Name[0] == '/')
 	    mi = rpmtsInitIterator(ts, RPMTAG_BASENAMES, Name, 0);
 	else
-	    mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, Name, 0);
+	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, Name, 0);
 
 	xx = rpmdbPruneIterator(mi,
 	    ts->removedPackages, ts->numRemovedPackages, 1);
@@ -818,28 +830,6 @@ static struct badDeps_s {
 /*@observer@*/ /*@null@*/ const char * pname;
 /*@observer@*/ /*@null@*/ const char * qname;
 } badDeps[] = {
-    { "libtermcap", "bash" },
-    { "modutils", "vixie-cron" },
-    { "ypbind", "yp-tools" },
-    { "ghostscript-fonts", "ghostscript" },
-    /* 7.2 only */
-    { "libgnomeprint15", "gnome-print" },
-    { "nautilus", "nautilus-mozilla" },
-    /* 7.1 only */
-    { "arts", "kdelibs-sound" },
-    /* 7.0 only */
-    { "pango-gtkbeta-devel", "pango-gtkbeta" },
-    { "XFree86", "Mesa" },
-    { "compat-glibc", "db2" },
-    { "compat-glibc", "db1" },
-    { "pam", "initscripts" },
-    { "initscripts", "sysklogd" },
-    /* 6.2 */
-    { "egcs-c++", "libstdc++" },
-    /* 6.1 */
-    { "pilot-link-devel", "pilot-link" },
-    /* 5.2 */
-    { "pam", "pamconfig" },
     { NULL, NULL }
 };
 #else
@@ -1244,9 +1234,11 @@ int rpmtsOrder(rpmts ts)
     int qlen;
     int i, j;
 
-#ifdef	DYING
+    /*
+     * XXX FIXME: this gets needlesly called twice on normal usage patterns,
+     * should track the need for generating the index somewhere
+     */
     rpmalMakeIndex(ts->addedPackages);
-#endif
 
     (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_ORDER), 0);
 
@@ -1294,14 +1286,12 @@ int rpmtsOrder(rpmts ts)
 	    switch (rpmteType(p)) {
 	    case TR_REMOVED:
 		/* Skip if not %preun/%postun requires or legacy prereq. */
-		if (isInstallPreReq(Flags)
-		 || !( isErasePreReq(Flags) || isLegacyPreReq(Flags) ) )
+		if (!( isErasePreReq(Flags) || isLegacyPreReq(Flags) ) )
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    case TR_ADDED:
 		/* Skip if not %pre/%post requires or legacy prereq. */
-		if (isErasePreReq(Flags)
-		 || !( isInstallPreReq(Flags) || isLegacyPreReq(Flags) ) )
+		if (!( isInstallPreReq(Flags) || isLegacyPreReq(Flags) ) )
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    }
@@ -1423,7 +1413,14 @@ rescan:
 	(void) rpmteSetDegree(q, 0);
 	tsbytes += rpmtePkgFileSize(q);
 
-	ordering[orderingCount] = rpmteAddedKey(q);
+	switch (rpmteType(q)) {
+	case TR_ADDED:
+            ordering[orderingCount] = rpmteAddedKey(q);
+            /*@switchbreak@*/ break;
+	case TR_REMOVED:
+            ordering[orderingCount] = RPMAL_NOMATCH;
+            /*@switchbreak@*/ break;
+        }
 	orderingCount++;
 	qlen--;
 	loopcheck--;
@@ -1620,7 +1617,7 @@ rescan:
 	    continue;
 
 	j = needle->orIndex;
-	if ((q = ts->order[j]) == NULL)
+	if ((q = ts->order[j]) == NULL || needle->pkgKey == RPMAL_NOMATCH)
 	    continue;
 
 	newOrder[newOrderCount++] = q;
