@@ -944,7 +944,7 @@ static int writeFile(/*@special@*/ /*@partial@*/ FSM_t fsm, int writeData)
     if (rc) goto exit;
 
     if (writeData && S_ISREG(st->st_mode)) {
-#if HAVE_MMAP
+#ifdef HAVE_MMAP
 	char * rdbuf = NULL;
 	void * mapped = (void *)-1;
 	size_t nmapped;
@@ -954,7 +954,7 @@ static int writeFile(/*@special@*/ /*@partial@*/ FSM_t fsm, int writeData)
 	if (rc) goto exit;
 
 	/* XXX unbuffered mmap generates *lots* of fdio debugging */
-#if HAVE_MMAP
+#ifdef HAVE_MMAP
 	nmapped = 0;
 	mapped = mmap(NULL, st->st_size, PROT_READ, MAP_SHARED, Fileno(fsm->rfd), 0);
 	if (mapped != (void *)-1) {
@@ -970,7 +970,7 @@ static int writeFile(/*@special@*/ /*@partial@*/ FSM_t fsm, int writeData)
 	left = st->st_size;
 
 	while (left) {
-#if HAVE_MMAP
+#ifdef HAVE_MMAP
 	  if (mapped != (void *)-1) {
 	    fsm->rdnb = nmapped;
 	  } else
@@ -988,7 +988,7 @@ static int writeFile(/*@special@*/ /*@partial@*/ FSM_t fsm, int writeData)
 	    left -= fsm->wrnb;
 	}
 
-#if HAVE_MMAP
+#ifdef HAVE_MMAP
 /*@-branchstate@*/
 	if (mapped != (void *)-1) {
 	    xx = msync(mapped, nmapped, MS_ASYNC);
@@ -1274,7 +1274,9 @@ static int fsmMkdirs(/*@special@*/ /*@partial@*/ FSM_t fsm)
     int dc = dnlCount(dnli);
     int rc = 0;
     int i;
+/*@-compdef@*/
     rpmts ts = fsmGetTs(fsm);
+/*@=compdef@*/
     rpmsx sx = rpmtsREContext(ts);
 
     fsm->path = NULL;
@@ -2125,6 +2127,11 @@ if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
 	/*@notreached@*/ break;
 
     case FSM_UNLINK:
+	if (fsm->mapFlags & CPIO_SBIT_CHECK) {
+	    struct stat stb;
+	    if (Lstat(fsm->path, &stb) == 0 && S_ISREG(stb.st_mode) && (stb.st_mode & 06000) != 0)
+		chmod(fsm->path, stb.st_mode & 0777);
+	}
 	rc = Unlink(fsm->path);
 	if (_fsm_debug && (stage & FSM_SYSCALL))
 	    rpmMessage(RPMMESS_DEBUG, " %8s (%s) %s\n", cur,
@@ -2180,7 +2187,7 @@ if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
 	    rpmMessage(RPMMESS_DEBUG, " %8s (%s, %s) %s\n", cur,
 		fsm->path, fsm->fcontext,
 		(rc < 0 ? strerror(errno) : ""));
-	if (rc < 0) rc = (errno == ENOTSUP ? 0 : CPIOERR_LSETFCON_FAILED);
+	if (rc < 0) rc = (errno == EOPNOTSUPP ? 0 : CPIOERR_LSETFCON_FAILED);
 	break;
     case FSM_CHOWN:
 	rc = chown(fsm->path, st->st_uid, st->st_gid);
@@ -2260,7 +2267,7 @@ if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
 		fsm->path, (rc < 0 ? strerror(errno) : ""));
 	if (rc < 0) {
 	    rc = (errno == ENOENT ? CPIOERR_ENOENT : CPIOERR_LSTAT_FAILED);
-	    memset(ost, 0, sizeof(*ost));
+	    memset(ost, 0, sizeof(*ost));	/* XXX s390x hackery */
 	}
 	break;
     case FSM_STAT:
@@ -2270,7 +2277,7 @@ if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
 		fsm->path, (rc < 0 ? strerror(errno) : ""));
 	if (rc < 0) {
 	    rc = (errno == ENOENT ? CPIOERR_ENOENT : CPIOERR_STAT_FAILED);
-	    memset(ost, 0, sizeof(*ost));
+	    memset(ost, 0, sizeof(*ost));	/* XXX s390x hackery */
 	}
 	break;
     case FSM_READLINK:
