@@ -463,9 +463,18 @@ rpmRC headerCheck(rpmts ts, const void * uh, size_t uc, const char ** msg)
 		siglen = blen + 1;
 	    }
 	    /*@switchbreak@*/ break;
-#ifdef	NOTYET
 	case RPMTAG_RSAHEADER:
-#endif
+	    if (vsflags & RPMVSF_NORSAHEADER)
+		/*@switchbreak@*/ break;
+	    if (entry->info.type != RPM_BIN_TYPE) {
+		(void) snprintf(buf, sizeof(buf), _("hdr RSA: BAD, not binary\n"));
+		goto exit;
+	    }
+/*@-boundswrite@*/
+	    *info = entry->info;	/* structure assignment */
+/*@=boundswrite@*/
+	    siglen = info->count;
+	    /*@switchbreak@*/ break;
 	case RPMTAG_DSAHEADER:
 	    if (vsflags & RPMVSF_NODSAHEADER)
 		/*@switchbreak@*/ break;
@@ -529,16 +538,16 @@ verifyinfo_exit:
     (void) rpmtsSetSig(ts, info->tag, info->type, sig, info->count);
 
     switch (info->tag) {
-#ifdef	NOTYET
     case RPMTAG_RSAHEADER:
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, info->count, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_WARNING,
-		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
+	    rpmMessage(RPMMESS_ERROR,
+		_("skipping header with unverifiable V%u signature\n"),
 		dig->signature.version);
 	    rpmtsCleanDig(ts);
-	    goto verifyinfo_exit;
+	    rc = RPMRC_FAIL;
+	    goto exit;
 	}
 
 	ildl[0] = htonl(ril);
@@ -546,7 +555,7 @@ verifyinfo_exit:
 	ildl[1] = htonl(ildl[1]);
 
 	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DIGEST), 0);
-	dig->hdrmd5ctx = rpmDigestInit(PGPHASHALGO_MD5, RPMDIGEST_NONE);
+	dig->hdrmd5ctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
 
 	b = (unsigned char *) header_magic;
 	nb = sizeof(header_magic);
@@ -570,16 +579,16 @@ verifyinfo_exit:
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DIGEST), dig->nbytes);
 
 	break;
-#endif
     case RPMTAG_DSAHEADER:
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, info->count, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_WARNING,
-		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
+	    rpmMessage(RPMMESS_ERROR,
+		_("skipping header with unverifiable V%u signature\n"),
 		dig->signature.version);
 	    rpmtsCleanDig(ts);
-	    goto verifyinfo_exit;
+	    rc = RPMRC_FAIL;
+	    goto exit;
 	}
 	/*@fallthrough@*/
     case RPMTAG_SHA1HEADER:
@@ -909,10 +918,10 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, siglen, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_WARNING,
-		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
-		dig->signature.version);
-	    rc = RPMRC_OK;	/* XXX return header w/o verify */
+	    rpmMessage(RPMMESS_ERROR,
+		_("skipping package %s with unverifiable V%u signature\n"),
+		fn, dig->signature.version);
+	    rc = RPMRC_FAIL;
 	    goto exit;
 	}
     {	void * uh = NULL;
@@ -922,10 +931,10 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	if (!headerGetEntry(h, RPMTAG_HEADERIMMUTABLE, &uht, &uh, &uhc))
 	    break;
 	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DIGEST), 0);
-	dig->md5ctx = rpmDigestInit(PGPHASHALGO_MD5, RPMDIGEST_NONE);
-	(void) rpmDigestUpdate(dig->md5ctx, header_magic, sizeof(header_magic));
+	dig->hdrmd5ctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
+	(void) rpmDigestUpdate(dig->hdrmd5ctx, header_magic, sizeof(header_magic));
 	dig->nbytes += sizeof(header_magic);
-	(void) rpmDigestUpdate(dig->md5ctx, uh, uhc);
+	(void) rpmDigestUpdate(dig->hdrmd5ctx, uh, uhc);
 	dig->nbytes += uhc;
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DIGEST), dig->nbytes);
 	rpmtsOp(ts, RPMTS_OP_DIGEST)->count--;	/* XXX one too many */
@@ -935,10 +944,10 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, siglen, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_WARNING,
-		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
-		dig->signature.version);
-	    rc = RPMRC_OK;	/* XXX return header w/o verify */
+	    rpmMessage(RPMMESS_ERROR,
+		_("skipping package %s with unverifiable V%u signature\n"), 
+		fn, dig->signature.version);
+	    rc = RPMRC_FAIL;
 	    goto exit;
 	}
 	/*@fallthrough@*/
@@ -967,10 +976,10 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	xx = pgpPrtPkts(sig, siglen, dig, (_print_pkts & rpmIsDebug()));
 
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_WARNING,
-		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
-		dig->signature.version);
-	    rc = RPMRC_OK;	/* XXX return header w/o verify */
+	    rpmMessage(RPMMESS_ERROR,
+		_("skipping package %s with unverifiable V%u signature\n"),
+		fn, dig->signature.version);
+	    rc = RPMRC_FAIL;
 	    goto exit;
 	}
 	/*@fallthrough@*/
@@ -992,17 +1001,23 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	/* XXX Steal the digest-in-progress from the file handle. */
 	for (i = fd->ndigests - 1; i >= 0; i--) {
 	    FDDIGEST_t fddig = fd->digests + i;
-	    if (fddig->hashctx == NULL)
-		continue;
-	    if (fddig->hashalgo == PGPHASHALGO_MD5) {
+	    if (fddig->hashctx != NULL)
+	    switch (fddig->hashalgo) {
+	    case PGPHASHALGO_MD5:
 		dig->md5ctx = fddig->hashctx;
 		fddig->hashctx = NULL;
-		continue;
-	    }
-	    if (fddig->hashalgo == PGPHASHALGO_SHA1) {
+		/*@switchbreak@*/ break;
+	    case PGPHASHALGO_SHA1:
+#if HAVE_BEECRYPT_API_H
+	    case PGPHASHALGO_SHA256:
+	    case PGPHASHALGO_SHA384:
+	    case PGPHASHALGO_SHA512:
+#endif
 		dig->sha1ctx = fddig->hashctx;
 		fddig->hashctx = NULL;
-		continue;
+		/*@switchbreak@*/ break;
+	    default:
+		/*@switchbreak@*/ break;
 	    }
 	}
 	break;
@@ -1051,4 +1066,43 @@ exit:
     sigh = rpmFreeSignature(sigh);
     return rc;
 }
+
+/**
+ * Check for supported payload format in header.
+ * @param h		header to check
+ * @return		RPMRC_OK if supported, RPMRC_FAIL otherwise
+ */
+rpmRC headerCheckPayloadFormat(Header h) {
+    rpmRC rc = RPMRC_FAIL;
+    int xx;
+    const char *payloadfmt = NULL;
+
+    xx = headerGetEntry(h, RPMTAG_PAYLOADFORMAT, NULL, 
+			(void **)&payloadfmt, NULL);
+    /* 
+     * XXX Ugh, rpm 3.x packages don't have payload format tag. Instead
+     * of blinly allowing, should check somehow (HDRID existence or... ?)
+     */
+    if (!payloadfmt)
+	return RPMRC_OK;
+
+    if (payloadfmt && strncmp(payloadfmt, "cpio", strlen("cpio")) == 0) {
+	rc = RPMRC_OK;
+    } else {
+        const char *nevra = hGetNEVRA(h, NULL);
+        if (payloadfmt && strncmp(payloadfmt, "drpm", strlen("drpm")) == 0) {
+            rpmMessage(RPMMESS_ERROR,
+                     _("%s is a Delta RPM and cannot be directly installed\n"),
+                     nevra);
+        } else {
+            rpmMessage(RPMMESS_ERROR, 
+                     _("Unsupported payload (%s) in package %s\n"),
+                     payloadfmt ? payloadfmt : "none", nevra);
+        } 
+        nevra = _free(nevra);
+    }
+    return rc;
+}
+
+
 /*@=bounds@*/
