@@ -4,8 +4,10 @@
  */
 
 #include "system.h"
+const char *__progname;
 
 #include <rpmcli.h>
+#include <fts.h>
 
 #include "debug.h"
 
@@ -61,6 +63,9 @@ extern int _rpmds_debug;
 /*@unchecked@*/
 extern int _rpmfi_debug;
 
+/*@unchecked@*/
+extern int _rpmgi_debug;
+
 /*@unchecked@*/
 extern int _rpmps_debug;
 
@@ -98,6 +103,10 @@ rpmQueryFlags rpmcliQueryFlags;
 /*@-exportheadervar@*/
 /*@unchecked@*/
 extern int _ftp_debug;
+/*@unchecked@*/
+extern int _av_debug;
+/*@unchecked@*/
+extern int _dav_debug;
 
 /*@unchecked@*/
 extern int noLibio;
@@ -212,6 +221,30 @@ static void rpmcliAllArgCallback( /*@unused@*/ poptContext con,
     /*@=branchstate@*/
 }
 
+/*@unchecked@*/
+int ftsOpts = 0;
+
+/*@unchecked@*/
+struct poptOption rpmcliFtsPoptTable[] = {
+ { "comfollow", '\0', POPT_BIT_SET,	&ftsOpts, FTS_COMFOLLOW,
+	N_("FTS_COMFOLLOW: follow command line symlinks"), NULL },
+ { "logical", '\0', POPT_BIT_SET,	&ftsOpts, FTS_LOGICAL,
+	N_("FTS_LOGICAL: logical walk"), NULL },
+ { "nochdir", '\0', POPT_BIT_SET,	&ftsOpts, FTS_NOCHDIR,
+	N_("FTS_NOCHDIR: don't change directories"), NULL },
+ { "nostat", '\0', POPT_BIT_SET,	&ftsOpts, FTS_NOSTAT,
+	N_("FTS_NOSTAT: don't get stat info"), NULL },
+ { "physical", '\0', POPT_BIT_SET,	&ftsOpts, FTS_PHYSICAL,
+	N_("FTS_PHYSICAL: physical walk"), NULL },
+ { "seedot", '\0', POPT_BIT_SET,	&ftsOpts, FTS_SEEDOT,
+	N_("FTS_SEEDOT: return dot and dot-dot"), NULL },
+ { "xdev", '\0', POPT_BIT_SET,		&ftsOpts, FTS_XDEV,
+	N_("FTS_XDEV: don't cross devices"), NULL },
+ { "whiteout", '\0', POPT_BIT_SET,	&ftsOpts, FTS_WHITEOUT,
+	N_("FTS_WHITEOUT: return whiteout information"), NULL },
+   POPT_TABLEEND
+};
+
 /*@-bitwisesigned -compmempass @*/
 /*@unchecked@*/
 struct poptOption rpmcliAllPoptTable[] = {
@@ -289,7 +322,11 @@ struct poptOption rpmcliAllPoptTable[] = {
  { "fsmthreads", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fsm_threads, -1,
 	N_("use threads for file state machine"), NULL},
  { "ftpdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_ftp_debug, -1,
-	N_("debug protocol data stream"), NULL},
+	N_("debug FTP/HTTP data stream"), NULL},
+ { "avdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_av_debug, -1,
+	N_("debug argv collections"), NULL},
+ { "davdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_dav_debug, -1,
+	N_("debug WebDAV data stream"), NULL},
  { "hdrdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_hdr_debug, -1,
 	NULL, NULL},
 #ifdef	DYING
@@ -312,6 +349,8 @@ struct poptOption rpmcliAllPoptTable[] = {
 	NULL, NULL},
  { "rpmfidebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmfi_debug, -1,
 	NULL, NULL},
+ { "rpmgidebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmgi_debug, -1,
+	NULL, NULL},
  { "rpmiodebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmio_debug, -1,
 	N_("debug rpmio I/O"), NULL},
  { "rpmpsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmps_debug, -1,
