@@ -62,25 +62,20 @@ int rpmVersionCompare(Header first, Header second)
 {
     const char * one, * two;
     int_32 * epochOne, * epochTwo;
+    static int_32 zero = 0;
     int rc;
 
     if (!headerGetEntry(first, RPMTAG_EPOCH, NULL, (void **) &epochOne, NULL))
-	epochOne = NULL;
+	epochOne = &zero;
     if (!headerGetEntry(second, RPMTAG_EPOCH, NULL, (void **) &epochTwo, NULL))
-	epochTwo = NULL;
+	epochTwo = &zero;
 
-    if (epochOne != NULL && epochTwo == NULL)
-	return 1;
-    else if (epochOne == NULL && epochTwo != NULL)
-	return -1;
-    else if (epochOne != NULL && epochTwo != NULL) {
 /*@-boundsread@*/
 	if (*epochOne < *epochTwo)
 	    return -1;
 	else if (*epochOne > *epochTwo)
 	    return 1;
 /*@=boundsread@*/
-    }
 
     rc = headerGetEntry(first, RPMTAG_VERSION, NULL, (void **) &one, NULL);
     rc = headerGetEntry(second, RPMTAG_VERSION, NULL, (void **) &two, NULL);
@@ -283,7 +278,10 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	goto exit;
     }
 
-    (void) rpmtsAddInstallElement(ts, h, NULL, 0, NULL);
+    if (rpmtsAddInstallElement(ts, h, NULL, 0, NULL)) {
+	rpmrc = RPMRC_FAIL;
+	goto exit;
+    }
 
     fi = rpmfiNew(ts, h, RPMTAG_BASENAMES, scareMem);
     h = headerFree(h);
@@ -490,6 +488,7 @@ static pid_t psmWait(rpmpsm psm)
     return psm->sq.reaped;
 }
 
+#ifdef WITH_LUA
 /**
  * Run internal Lua script.
  */
@@ -572,6 +571,7 @@ static rpmRC runLuaScript(rpmpsm psm, Header h, const char *sln,
 
     return rc;
 }
+#endif
 
 /**
  */
@@ -632,24 +632,24 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     if (progArgv == NULL && script == NULL)
 	return rc;
 
+    /* XXX FIXME: except for %verifyscript, rpmteNEVR can be used. */
+    xx = headerNVR(h, &n, &v, &r);
+    xx = hge(h, RPMTAG_ARCH, NULL, (void **) &a, NULL);
+
     if (progArgv && strcmp(progArgv[0], "<lua>") == 0) {
+#ifdef WITH_LUA
 	rpmMessage(RPMMESS_DEBUG,
 		_("%s: %s(%s-%s-%s.%s) running <lua> scriptlet.\n"),
 		psm->stepName, tag2sln(psm->scriptTag), n, v, r, a);
 	return runLuaScript(psm, h, sln, progArgc, progArgv,
 			    script, arg1, arg2);
+#else
+	return RPMRC_FAIL;
+#endif
     }
 
     psm->sq.reaper = 1;
 
-    /* XXX FIXME: except for %verifyscript, rpmteNEVR can be used. */
-    xx = headerNVR(h, &n, &v, &r);
-    xx = hge(h, RPMTAG_ARCH, NULL, (void **) &a, NULL);
-
-    /* XXX bash must have functional libtermcap.so.2 */
-    if (!strcmp(n, "libtermcap"))
-	ldconfig_done = 0;
-
     /*
      * If a successor node, and ldconfig was just run, don't bother.
      */
@@ -827,13 +827,8 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    /* make sure we don't close stdin/stderr/stdout by mistake! */
 	    if (ofdno > STDERR_FILENO && ofdno != sfdno)
 		xx = Fclose (out);
-	    if (sfdno > STDERR_FILENO)
+	    if (sfdno > STDERR_FILENO && ofdno != sfdno)
 		xx = Fclose (scriptFd);
-	    else {
-/*@-usereleased@*/
-		xx = Fclose(out);
-/*@=usereleased@*/
-	    }
 	}
 
 	{   const char *ipath = rpmExpand("PATH=%{_install_script_path}", NULL);
@@ -910,6 +905,12 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     }
     /*@=branchstate@*/
 
+    if (psm->sq.child == (pid_t)-1) {
+	rpmError(RPMERR_FORK, _("Couldn't fork %s: %s\n"), sln, strerror(errno));
+	rc = RPMRC_FAIL;
+	goto exit;
+    }
+
     (void) psmWait(psm);
 
   /* XXX filter order dependent multilib "other" arch helper error. */
@@ -921,13 +922,20 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	rc = RPMRC_FAIL;
     } else
     if (!WIFEXITED(psm->sq.status) || WEXITSTATUS(psm->sq.status)) {
+      if (WIFSIGNALED(psm->sq.status)) {
+        rpmError(RPMERR_SCRIPT,
+                 _("%s(%s-%s-%s.%s) scriptlet failed, signal %d\n"),
+                 sln, n, v, r, a, WTERMSIG(psm->sq.status));
+      } else {
 	rpmError(RPMERR_SCRIPT,
 		_("%s(%s-%s-%s.%s) scriptlet failed, exit status %d\n"),
 		sln, n, v, r, a, WEXITSTATUS(psm->sq.status));
+      }
 	rc = RPMRC_FAIL;
     }
   }
 
+exit:
     if (freePrefixes) prefixes = hfd(prefixes, ipt);
 
     xx = Fclose(out);	/* XXX dup'd STDOUT_FILENO */
@@ -1459,7 +1467,7 @@ assert(psm->mi == NULL);
 		fi->striplen = (xx ? strlen(p) + 1 : 1);
 	    }
 	    fi->mapflags =
-		CPIO_MAP_PATH | CPIO_MAP_MODE | CPIO_MAP_UID | CPIO_MAP_GID;
+		CPIO_MAP_PATH | CPIO_MAP_MODE | CPIO_MAP_UID | CPIO_MAP_GID | (fi->mapflags & CPIO_SBIT_CHECK);
 	
 	    if (headerIsEntry(fi->h, RPMTAG_ORIGBASENAMES))
 		rpmfiBuildFNames(fi->h, RPMTAG_ORIGBASENAMES, &fi->apath, NULL);
