@@ -116,6 +116,10 @@ static int copyFile(FD_t *sfdp, const char **sfnp,
 	rpmError(RPMERR_FREAD, _("%s: Fread failed: %s\n"), *sfnp, Fstrerror(*sfdp));
 	goto exit;
     }
+    if (Fflush(*tfdp) != 0) {
+	rpmError(RPMERR_FWRITE, _("%s: Fflush failed: %s\n"), *tfnp,
+	    Fstrerror(*tfdp));
+    }
 
     rc = 0;
 
@@ -306,6 +310,12 @@ static int rpmReSign(/*@unused@*/ rpmts ts,
 	    xx = getSignid(sigh, sigtag, oldsignid);
 
 	    switch (sigtag) {
+	    case RPMSIGTAG_DSA:
+		xx = headerRemoveEntry(sigh, RPMSIGTAG_GPG);
+		/*@switchbreak@*/ break;
+	    case RPMSIGTAG_RSA:
+		xx = headerRemoveEntry(sigh, RPMSIGTAG_PGP);
+		/*@switchbreak@*/ break;
 	    case RPMSIGTAG_GPG:
 		xx = headerRemoveEntry(sigh, RPMSIGTAG_DSA);
 		/*@fallthrough@*/
@@ -661,6 +671,9 @@ static int readFile(FD_t fd, const char * fn, pgpDig dig)
 	    dig->hdrsha1ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
 	    (void) rpmDigestUpdate(dig->hdrsha1ctx, header_magic, sizeof(header_magic));
 	    (void) rpmDigestUpdate(dig->hdrsha1ctx, uh, uhc);
+	    dig->hdrmd5ctx = rpmDigestInit(dig->signature.hash_algo, RPMDIGEST_NONE);
+	    (void) rpmDigestUpdate(dig->hdrmd5ctx, header_magic, sizeof(header_magic));
+	    (void) rpmDigestUpdate(dig->hdrmd5ctx, uh, uhc);
 	    uh = headerFreeData(uh, uht);
 	}
 	h = headerFree(h);
@@ -677,19 +690,25 @@ static int readFile(FD_t fd, const char * fn, pgpDig dig)
     /* XXX Steal the digest-in-progress from the file handle. */
     for (i = fd->ndigests - 1; i >= 0; i--) {
 	FDDIGEST_t fddig = fd->digests + i;
-	if (fddig->hashctx == NULL)
-	    continue;
-	if (fddig->hashalgo == PGPHASHALGO_MD5) {
+	if (fddig->hashctx != NULL)
+	switch (fddig->hashalgo) {
+	case PGPHASHALGO_MD5:
 assert(dig->md5ctx == NULL);
 	    dig->md5ctx = fddig->hashctx;
 	    fddig->hashctx = NULL;
-	    continue;
-	}
-	if (fddig->hashalgo == PGPHASHALGO_SHA1) {
+	    /*@switchbreak@*/ break;
+	case PGPHASHALGO_SHA1:
+#if HAVE_BEECRYPT_API_H
+	case PGPHASHALGO_SHA256:
+	case PGPHASHALGO_SHA384:
+	case PGPHASHALGO_SHA512:
+#endif
 assert(dig->sha1ctx == NULL);
 	    dig->sha1ctx = fddig->hashctx;
 	    fddig->hashctx = NULL;
-	    continue;
+	    /*@switchbreak@*/ break;
+	default:
+	    /*@switchbreak@*/ break;
 	}
     }
 
@@ -782,6 +801,21 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 		sigtag = RPMSIGTAG_SHA1;	/* XXX never happens */
 	}
 
+	dig = rpmtsDig(ts);
+	sigp = rpmtsSignature(ts);
+
+	/* XXX RSA needs the hash_algo, so decode early. */
+	if (sigtag == RPMSIGTAG_RSA || sigtag == RPMSIGTAG_PGP) {
+	    xx = headerGetEntry(sigh, sigtag, &sigtype, &sig, &siglen);
+	    xx = pgpPrtPkts(sig, siglen, dig, 0);
+	    sig = headerFreeData(sig, sigtype);
+	    /* XXX assume same hash_algo in header-only and header+payload */
+	    if ((headerIsEntry(sigh, RPMSIGTAG_PGP)
+	      || headerIsEntry(sigh, RPMSIGTAG_PGP5))
+	     && dig->signature.hash_algo != PGPHASHALGO_MD5)
+		fdInitDigest(fd, dig->signature.hash_algo, 0);
+	}
+
 	if (headerIsEntry(sigh, RPMSIGTAG_PGP)
 	||  headerIsEntry(sigh, RPMSIGTAG_PGP5)
 	||  headerIsEntry(sigh, RPMSIGTAG_MD5))
@@ -789,9 +823,6 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 	if (headerIsEntry(sigh, RPMSIGTAG_GPG))
 	    fdInitDigest(fd, PGPHASHALGO_SHA1, 0);
 
-	dig = rpmtsDig(ts);
-	sigp = rpmtsSignature(ts);
-
 	/* Read the file, generating digest(s) on the fly. */
 	if (dig == NULL || sigp == NULL || readFile(fd, fn, dig)) {
 	    res++;
@@ -831,9 +862,10 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 
 		if (sigp->version != 3 && sigp->version != 4) {
 		    rpmError(RPMERR_SIGVFY,
-		_("only V3 or V4 signatures can be verified, skipping V%u signature\n"),
-			sigp->version);
-		    continue;
+		_("skipping package %s with unverifiable V%u signature\n"),
+			fn, sigp->version);
+		    res++;
+		    goto exit;
 		}
 		/*@switchbreak@*/ break;
 	    case RPMSIGTAG_SHA1:
