@@ -445,6 +445,9 @@ typedef enum rpmTag_e {
     RPMTAG_ENHANCESVERSION	= 1160,	/* s[] extension placeholder */
     RPMTAG_ENHANCESFLAGS	= 1161,	/* i   extension placeholder */
     RPMTAG_PRIORITY		= 1162, /* i   extension placeholder */
+    RPMTAG_CVSID		= 1163, /* s */
+#define	RPMTAG_SVNID	RPMTAG_CVSID	/* s */
+
 /*@-enummemuse@*/
     RPMTAG_FIRSTFREE_TAG	/*!< internal */
 /*@=enummemuse@*/
@@ -890,6 +893,13 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmpsm_s * rpmpsm;
 int headerVerifyInfo(int il, int dl, const void * pev, void * iv, int negate)
 	/*@modifies *iv @*/;
 
+/**
+ * Check for supported payload format in header.
+ * @param h		header to check
+ * @return		RPMRC_OK if supported, RPMRC_FAIL otherwise
+ */
+rpmRC headerCheckPayloadFormat(Header h);
+
 /** 
  * Check header consistency, performing headerGetEntry() the hard way.
  *  
@@ -1058,9 +1068,17 @@ void rpmShowRpmlibProvides(FILE * fp)
 /*@}*/
 
 /**
- * Return name of tag from value.
+ * Return  tag data type from value.
  * @param tag		tag value
- * @return		name of tag
+ * @return		tag data type, RPM_NULL_TYPE on not found.
+ */
+int tagType(int tag)
+	/*@*/;
+
+/**
+ * Return tag name from value.
+ * @param tag		tag value
+ * @return		tag name, "(unknown)" on not found
  */
 /*@-redecl@*/
 /*@observer@*/ extern const char *const tagName(int tag)
@@ -1068,9 +1086,9 @@ void rpmShowRpmlibProvides(FILE * fp)
 /*@=redecl@*/
 
 /**
- * Return value of tag from name.
+ * Return tag value from name.
  * @param tagstr	name of tag
- * @return		tag value
+ * @return		tag value, -1 on not found
  */
 int tagValue(const char * tagstr)
 	/*@*/;
