@@ -953,20 +953,28 @@ static inline int RPMClass(void)
 {
 	int cpu;
 	unsigned int tfms, junk, cap, capamd;
+	struct sigaction oldsa;
 	
+	sigaction(SIGILL, NULL, &oldsa);
 	signal(SIGILL, model3);
 	
-	if (sigsetjmp(jenv, 1))
+	if (sigsetjmp(jenv, 1)) {
+		sigaction(SIGILL, &oldsa, NULL);
 		return 3;
+	}
 		
-	if (cpuid_eax(0x000000000)==0)
+	if (cpuid_eax(0x000000000)==0) {
+		sigaction(SIGILL, &oldsa, NULL);
 		return 4;
+	}
 
 	cpuid(0x00000001, &tfms, &junk, &junk, &cap);
 	cpuid(0x80000001, &junk, &junk, &junk, &capamd);
 	
 	cpu = (tfms>>8)&15;
 	
+	sigaction(SIGILL, &oldsa, NULL);
+
 	if (cpu < 6)
 		return cpu;
 		
@@ -1326,27 +1334,6 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 	}
 #	endif
 
-#	if defined(__linux__) && defined(__powerpc__)
-	{
-	    unsigned pvr = 0;
-	    __sighandler_t oldh = signal(SIGILL, mfspr_ill);
-	    if (setjmp(mfspr_jmpbuf) == 0) {
-		__asm__ __volatile__ ("mfspr %0, 287" : "=r" (pvr));
-	    }
-	    signal(SIGILL, oldh);
-
-	    if ( pvr ) {
-		pvr >>= 16;
-		if ( pvr >= 0x40)
-		    strcpy(un.machine, "ppcpseries");
-		else if ( (pvr == 0x36) || (pvr == 0x37) )
-		    strcpy(un.machine, "ppciseries");
-		else
-		    strcpy(un.machine, "ppc");
-	    }
-	}
-#	endif
-
 	/* the uname() result goes through the arch_canon table */
 	canon = lookupInCanonTable(un.machine,
 				   tables[RPM_MACHTABLE_INSTARCH].canons,
@@ -1453,7 +1440,8 @@ static void rpmSetVarArch(int var, const char * val, const char * arch)
 	}
     }
 
-    next->value = xstrdup(val);		/* XXX memory leak, hard to plug */
+    next->value = _free(next->value);
+    next->value = xstrdup(val);
     next->arch = (arch ? xstrdup(arch) : NULL);
 }
 
@@ -1573,7 +1561,7 @@ static void getMachineInfo(int type, /*@null@*/ /*@out@*/ const char ** name,
 
 	if (tables[currTables[type]].hasCanon) {
 	    rpmMessage(RPMMESS_WARNING, _("Unknown system: %s\n"), current[type]);
-	    rpmMessage(RPMMESS_WARNING, _("Please contact rpm-list@redhat.com\n"));
+	    rpmMessage(RPMMESS_WARNING, _("Please contact rpm-maint@lists.rpm.org\n"));
 	}
     }
 }
@@ -1883,7 +1871,9 @@ int rpmReadConfigFiles(const char * file, const char * target)
     }
 
     /* Force Lua state initialization */
+#ifdef WITH_LUA
     (void)rpmluaGetPrintBuffer(NULL);
+#endif
 
     return 0;
 }
