@@ -236,7 +236,7 @@ static int isArch(const char * arch)
 	"m68k",
 	"rs6000",
 	"ia64",
-	"armv3l", "armv4b", "armv4l",
+	"armv3l", "armv4b", "armv4l", "armv4tl", "armv5tel", "armv5tejl", "armv6l",
 	"s390", "i370", "s390x",
 	"sh", "xtensa",
 	"noarch",
@@ -1475,6 +1475,11 @@ uint_32 rpmtsSetColor(rpmts ts, uint_32 color)
     return ocolor;
 }
 
+uint_32 rpmtsPrefColor(rpmts ts)
+{
+    return (ts != NULL ? ts->prefcolor : 0);
+}
+
 rpmop rpmtsOp(rpmts ts, rpmtsOpX opx)
 {
     rpmop op = NULL;
@@ -1554,6 +1559,7 @@ rpmts rpmtsCreate(void)
     ts->delta = 5;
 
     ts->color = rpmExpandNumeric("%{?_transaction_color}");
+    ts->prefcolor = rpmExpandNumeric("%{?_prefer_color}")?:2;
 
     ts->numRemovedPackages = 0;
     ts->allocedRemovedPackages = ts->delta;
