@@ -122,6 +122,7 @@ static int handleInstInstalledFiles(const rpmts ts,
 	/*@modifies ts, fi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     uint_32 tscolor = rpmtsColor(ts);
+    uint_32 prefcolor = rpmtsPrefColor(ts);
     uint_32 otecolor, tecolor;
     uint_32 oFColor, FColor;
     const char * altNEVR = NULL;
@@ -187,6 +188,13 @@ static int handleInstInstalledFiles(const rpmts ts,
 	if (XFA_SKIPPING(fi->actions[fileNum]))
 	    continue;
 
+	if (!(fi->mapflags & CPIO_SBIT_CHECK)) {
+	    int_16 omode = rpmfiFMode(otherFi);
+	    if (S_ISREG(omode) && (omode & 06000) != 0) {
+		fi->mapflags |= CPIO_SBIT_CHECK;
+	    }
+	}
+
 	if (rpmfiCompare(otherFi, fi)) {
 	    int rConflicts;
 
@@ -194,11 +202,11 @@ static int handleInstInstalledFiles(const rpmts ts,
 	    /* Resolve file conflicts to prefer Elf64 (if not forced). */
 	    if (tscolor != 0 && FColor != 0 && FColor != oFColor)
 	    {
-		if (oFColor & 0x2) {
+		if (oFColor & prefcolor) {
 		    fi->actions[fileNum] = FA_SKIPCOLOR;
 		    rConflicts = 0;
 		} else
-		if (FColor & 0x2) {
+		if (FColor & prefcolor) {
 		    fi->actions[fileNum] = FA_CREATE;
 		    rConflicts = 0;
 		}
@@ -453,6 +461,7 @@ static void handleOverlappedFiles(const rpmts ts,
     if (fi != NULL)
     while ((i = rpmfiNext(fi)) >= 0) {
 	uint_32 tscolor = rpmtsColor(ts);
+	uint_32 prefcolor = rpmtsPrefColor(ts);
 	uint_32 oFColor, FColor;
 	struct fingerPrint_s * fiFps;
 	int otherPkgNum, otherFileNum;
@@ -565,19 +574,19 @@ assert(otherFi != NULL);
 		rConflicts = reportConflicts;
 		/* Resolve file conflicts to prefer Elf64 (if not forced) ... */
 		if (tscolor != 0) {
-		    if (FColor & 0x2) {
-			/* ... last Elf64 file is installed ... */
+		    if (FColor & prefcolor) {
+			/* ... last file of preferred colour is installed ... */
 			if (!XFA_SKIPPING(fi->actions[i])) {
 			    /* XXX static helpers are order dependent. Ick. */
 			    if (strcmp(fn, "/usr/sbin/libgcc_post_upgrade")
 			     && strcmp(fn, "/usr/sbin/glibc_post_upgrade"))
-				otherFi->actions[otherFileNum] = FA_SKIP;
+				otherFi->actions[otherFileNum] = FA_SKIPCOLOR;
 			}
 			fi->actions[i] = FA_CREATE;
 			rConflicts = 0;
 		    } else
-		    if (oFColor & 0x2) {
-			/* ... first Elf64 file is installed ... */
+		    if (oFColor & prefcolor) {
+			/* ... first file of preferred colour is installed ... */
 			if (XFA_SKIPPING(fi->actions[i]))
 			    otherFi->actions[otherFileNum] = FA_CREATE;
 			fi->actions[i] = FA_SKIPCOLOR;
@@ -636,8 +645,6 @@ assert(otherFi != NULL);
 	    }
 		
 	    /* Here is a pre-existing modified config file that needs saving. */
-	    /* XXX avoid md5 on sparse /var/log/lastlog file. */
-	    if (strcmp(fn, "/var/log/lastlog"))
 	    {	char md5sum[50];
 		const unsigned char * MD5 = rpmfiMD5(fi);
 		if (!domd5(fn, md5sum, 0, NULL) && memcmp(MD5, md5sum, 16)) {
@@ -815,6 +822,9 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 		    /*@innercontinue@*/ continue;
 		if (strncmp(dn, *nsp, dnlen))
 		    /*@innercontinue@*/ continue;
+		/* Insure that only the netsharedpath basename is compared. */
+		if ((s = strchr((*nsp) + dnlen, '/')) != NULL && s[1] != '\0')
+		    /*@innercontinue@*/ continue;
 		if (strncmp(bn, (*nsp) + dnlen, bnlen))
 		    /*@innercontinue@*/ continue;
 		len = dnlen + bnlen;
@@ -1565,7 +1575,7 @@ rpmMessage(RPMMESS_DEBUG, _("sanity checking %d elements\n"), rpmtsNElements(ts)
 
     /* Run pre-transaction scripts, but only if there are no known
      * problems up to this point. */
-    if (!((rpmtsFlags(ts) & RPMTRANS_FLAG_BUILD_PROBS)
+    if (!((rpmtsFlags(ts) & (RPMTRANS_FLAG_BUILD_PROBS|RPMTRANS_FLAG_TEST))
      	  || (ts->probs->numProblems &&
 		(okProbs == NULL || rpmpsTrim(ts->probs, okProbs))))) {
 	rpmMessage(RPMMESS_DEBUG, _("running pre-transaction scripts\n"));
@@ -1677,8 +1687,11 @@ rpmMessage(RPMMESS_DEBUG, _("computing %d file fingerprints\n"), totalFileCount)
 	const char * rootDir = rpmtsRootDir(ts);
 	xx = chdir("/");
 	/*@-superuser -noeffect @*/
-	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
+	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/') {
+	    /* opening db before chroot not optimal, see rhbz#103852 c#3 */
+	    xx = rpmdbOpenAll(ts->rdb);
 	    xx = chroot(rootDir);
+	}
 	/*@=superuser =noeffect @*/
 	(void) rpmtsSetChrootDone(ts, 1);
     }
@@ -1843,6 +1856,20 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 	case TR_REMOVED:
 	    /*@switchbreak@*/ break;
 	}
+	/* check for s-bit files to be removed */
+	if (rpmteType(p) == TR_REMOVED) {
+	    fi = rpmfiInit(fi, 0);
+	    while ((i = rpmfiNext(fi)) >= 0) {
+		int_16 mode;
+		if (XFA_SKIPPING(fi->actions[i]))
+		    continue;
+		(void) rpmfiSetFX(fi, i);
+		mode = rpmfiFMode(fi);
+		if (S_ISREG(mode) && (mode & 06000) != 0) {
+		    fi->mapflags |= CPIO_SBIT_CHECK;
+		}
+	    }
+	}
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_FINGERPRINT), fc);
     }
     pi = rpmtsiFree(pi);
@@ -2088,10 +2115,26 @@ assert(psm != NULL);
 		{
 		    char * fstates = fi->fstates;
 		    fileAction * actions = fi->actions;
+		    sharedFileInfo replaced = fi->replaced;
+		    int mapflags = fi->mapflags;
 		    rpmte savep;
+		    int numShared = 0;
+
+		    if (replaced != NULL) {
+                        for (replaced; replaced->otherPkg; replaced++) {
+                            numShared++;
+                        }
+                        if (numShared > 0) {
+                            replaced = xcalloc(numShared + 1, 
+                                               sizeof(*fi->replaced));
+                            memcpy(replaced, fi->replaced, 
+                                   sizeof(*fi->replaced) * (numShared + 1));
+                        }
+                    }
 
 		    fi->fstates = NULL;
 		    fi->actions = NULL;
+		    fi->replaced = NULL;
 /*@-nullstate@*/ /* FIX: fi->actions is NULL */
 		    fi = rpmfiFree(fi);
 /*@=nullstate@*/
@@ -2106,6 +2149,10 @@ assert(psm != NULL);
 			fi->fstates = fstates;
 			fi->actions = _free(fi->actions);
 			fi->actions = actions;
+                        if (replaced != NULL)
+                            fi->replaced = replaced;
+			if (mapflags & CPIO_SBIT_CHECK)
+			    fi->mapflags |= CPIO_SBIT_CHECK;
 			p->fi = fi;
 		    }
 		}
@@ -2273,68 +2320,70 @@ assert(psm != NULL);
     if (rollbackOnFailure && rollbackTransaction != NULL)
 	rollbackTransaction = rpmtsFree(rollbackTransaction);
 
-    rpmMessage(RPMMESS_DEBUG, _("running post-transaction scripts\n"));
-    pi = rpmtsiInit(ts);
-    while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
-	int haspostscript;
+    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)) {
+	rpmMessage(RPMMESS_DEBUG, _("running post-transaction scripts\n"));
+	pi = rpmtsiInit(ts);
+	while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
+	    int haspostscript;
 
-	if ((fi = rpmtsiFi(pi)) == NULL)
-	    continue;	/* XXX can't happen */
+	    if ((fi = rpmtsiFi(pi)) == NULL)
+		continue;	/* XXX can't happen */
 
-	haspostscript = (fi->posttrans != NULL ? 1 : 0);
-	p->fi = rpmfiFree(p->fi);
+	    haspostscript = (fi->posttrans != NULL ? 1 : 0);
+	    p->fi = rpmfiFree(p->fi);
 
-	/* If no post-transaction script, then don't bother. */
-	if (haspostscript)
-	    continue;
+	    /* If no post-transaction script, then don't bother. */
+	    if (!haspostscript)
+		continue;
 
-	p->fd = ts->notify(p->h, RPMCALLBACK_INST_OPEN_FILE, 0, 0,
-			rpmteKey(p), ts->notifyData);
-	p->h = NULL;
-	if (rpmteFd(p) != NULL) {
-	    rpmVSFlags ovsflags = rpmtsVSFlags(ts);
-	    rpmVSFlags vsflags = ovsflags | RPMVSF_NEEDPAYLOAD;
-	    rpmRC rpmrc;
-	    ovsflags = rpmtsSetVSFlags(ts, vsflags);
-	    rpmrc = rpmReadPackageFile(ts, rpmteFd(p),
-			rpmteNEVR(p), &p->h);
-	    vsflags = rpmtsSetVSFlags(ts, ovsflags);
-	    switch (rpmrc) {
-	    default:
-		p->fd = ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE,
-				0, 0, rpmteKey(p), ts->notifyData);
-		p->fd = NULL;
-		/*@switchbreak@*/ break;
-	    case RPMRC_NOTTRUSTED:
-	    case RPMRC_NOKEY:
-	    case RPMRC_OK:
-		/*@switchbreak@*/ break;
+	    p->fd = ts->notify(p->h, RPMCALLBACK_INST_OPEN_FILE, 0, 0,
+			    rpmteKey(p), ts->notifyData);
+	    p->h = NULL;
+	    if (rpmteFd(p) != NULL) {
+		rpmVSFlags ovsflags = rpmtsVSFlags(ts);
+		rpmVSFlags vsflags = ovsflags | RPMVSF_NEEDPAYLOAD;
+		rpmRC rpmrc;
+		ovsflags = rpmtsSetVSFlags(ts, vsflags);
+		rpmrc = rpmReadPackageFile(ts, rpmteFd(p),
+			    rpmteNEVR(p), &p->h);
+		vsflags = rpmtsSetVSFlags(ts, ovsflags);
+		switch (rpmrc) {
+		default:
+		    p->fd = ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE,
+				    0, 0, rpmteKey(p), ts->notifyData);
+		    p->fd = NULL;
+		    /*@switchbreak@*/ break;
+		case RPMRC_NOTTRUSTED:
+		case RPMRC_NOKEY:
+		case RPMRC_OK:
+		    /*@switchbreak@*/ break;
+		}
 	    }
-	}
 
-	if (rpmteFd(p) != NULL) {
-	    p->fi = rpmfiNew(ts, p->h, RPMTAG_BASENAMES, 1);
-	    if (p->fi != NULL)	/* XXX can't happen */
-		p->fi->te = p;
+	    if (rpmteFd(p) != NULL) {
+		p->fi = rpmfiNew(ts, p->h, RPMTAG_BASENAMES, 1);
+		if (p->fi != NULL)	/* XXX can't happen */
+		    p->fi->te = p;
 /*@-compdef -usereleased@*/	/* p->fi->te undefined */
-	    psm = rpmpsmNew(ts, p, p->fi);
+		psm = rpmpsmNew(ts, p, p->fi);
 /*@=compdef =usereleased@*/
 assert(psm != NULL);
-	    psm->scriptTag = RPMTAG_POSTTRANS;
-	    psm->progTag = RPMTAG_POSTTRANSPROG;
-	    xx = rpmpsmStage(psm, PSM_SCRIPT);
-	    psm = rpmpsmFree(psm);
+		psm->scriptTag = RPMTAG_POSTTRANS;
+		psm->progTag = RPMTAG_POSTTRANSPROG;
+		xx = rpmpsmStage(psm, PSM_SCRIPT);
+		psm = rpmpsmFree(psm);
 
 /*@-noeffectuncon -compdef -usereleased @*/
-	    (void) ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
-			      rpmteKey(p), ts->notifyData);
+		(void) ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
+				  rpmteKey(p), ts->notifyData);
 /*@=noeffectuncon =compdef =usereleased @*/
-	    p->fd = NULL;
-	    p->fi = rpmfiFree(p->fi);
-	    p->h = headerFree(p->h);
+		p->fd = NULL;
+		p->fi = rpmfiFree(p->fi);
+		p->h = headerFree(p->h);
+	    }
 	}
+	pi = rpmtsiFree(pi);
     }
-    pi = rpmtsiFree(pi);
 
     rpmtsFreeLock(lock);
 
