@@ -333,7 +333,8 @@ dbiIndex dbiOpen(rpmdb db, rpmTag rpmtag, /*@unused@*/ unsigned int flags)
     }
 
 /* We don't ever _REQUIRE_ conversion... */
-#ifdef	SQLITE_HACK
+#define	SQLITE_HACK
+#ifdef	SQLITE_HACK_XXX
     /* Require conversion. */
     if (rc && _dbapi_wanted >= 0 && _dbapi != _dbapi_wanted && _dbapi_wanted == _dbapi_rebuild) {
 	rc = (_rebuildinprogress ? 0 : 1);
@@ -794,6 +795,22 @@ static struct rpmdb_s dbTemplate = {
 };
 /*@=fullinitblock@*/
 
+static int isTemporaryDB(int rpmtag) 
+{
+    int rc = 0;
+    switch (rpmtag) {
+    case RPMDBI_AVAILABLE:
+    case RPMDBI_ADDED:
+    case RPMDBI_REMOVED:
+    case RPMDBI_DEPENDS:
+	rc = 1;
+	break;
+    default:
+	break;
+    }
+    return rc;
+}
+
 int rpmdbOpenAll(rpmdb db)
 {
     int dbix;
@@ -805,6 +822,9 @@ int rpmdbOpenAll(rpmdb db)
     for (dbix = 0; dbix < dbiTagsMax; dbix++) {
 	if (db->_dbi[dbix] != NULL)
 	    continue;
+	/* Filter out temporary databases */
+	if (isTemporaryDB(dbiTags[dbix])) 
+	    continue;
 	(void) dbiOpen(db, dbiTags[dbix], db->db_flags);
     }
     return rc;
@@ -905,6 +925,8 @@ int rpmdbSync(rpmdb db)
 	int xx;
 	if (db->_dbi[dbix] == NULL)
 	    continue;
+	if (db->_dbi[dbix]->dbi_no_dbsync)
+	    continue;
     	xx = dbiSync(db->_dbi[dbix], 0);
 	if (xx && rc == 0) rc = xx;
     }
@@ -1037,16 +1059,8 @@ static int openDatabase(/*@null@*/ const char * prefix,
 	    int rpmtag;
 
 	    /* Filter out temporary databases */
-	    switch ((rpmtag = dbiTags[dbix])) {
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB((rpmtag = dbiTags[dbix])))
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
-	    default:
-		/*@switchbreak@*/ break;
-	    }
 
 	    dbi = dbiOpen(db, rpmtag, 0);
 	    if (dbi == NULL) {
@@ -2667,14 +2681,11 @@ memset(data, 0, sizeof(*data));
 /*@=boundsread@*/
 
 	    /*@-branchstate@*/
-	    switch (rpmtag) {
 	    /* Filter out temporary databases */
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB(rpmtag)) 
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
+
+	    switch (rpmtag) {
 	    case RPMDBI_PACKAGES:
 		dbi = dbiOpen(db, rpmtag, 0);
 		if (dbi == NULL)	/* XXX shouldn't happen */
@@ -3068,14 +3079,11 @@ memset(data, 0, sizeof(*data));
 	    rpmtag = dbiTags[dbix];
 /*@=boundsread@*/
 
-	    switch (rpmtag) {
 	    /* Filter out temporary databases */
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB(rpmtag)) 
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
+
+	    switch (rpmtag) {
 	    case RPMDBI_PACKAGES:
 		dbi = dbiOpen(db, rpmtag, 0);
 		if (dbi == NULL)	/* XXX shouldn't happen */
@@ -3360,13 +3368,6 @@ static struct skipDir_s {
 /*@observer@*/ /*@null@*/
     const char * dn;
 } skipDirs[] = {
-    _skip("/usr/share/zoneinfo"),
-    _skip("/usr/share/locale"),
-    _skip("/usr/share/i18n"),
-    _skip("/usr/share/doc"),
-    _skip("/usr/lib/locale"),
-    _skip("/usr/src"),
-    _skip("/lib/modules"),
     { 0, NULL }
 };
 
@@ -3401,11 +3402,11 @@ DBT * data;
     Header h;
     int i, xx;
 
-    if (db == NULL) return 0;
+    if (db == NULL) return 1;
 
     mi = rpmdbInitIterator(db, RPMTAG_BASENAMES, NULL, 0);
     if (mi == NULL)	/* XXX should  never happen */
-	return 0;
+	return 1;
 
 key = &mi->mi_key;
 data = &mi->mi_data;
@@ -3657,16 +3658,8 @@ static int rpmdbMoveDatabase(const char * prefix,
 	    int rpmtag;
 
 	    /* Filter out temporary databases */
-	    switch ((rpmtag = dbiTags[i])) {
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB((rpmtag = dbiTags[i])))
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
-	    default:
-		/*@switchbreak@*/ break;
-	    }
 
 	    base = tagName(rpmtag);
 	    sprintf(ofilename, "%s/%s/%s", prefix, olddbpath, base);
@@ -3712,7 +3705,7 @@ static int rpmdbMoveDatabase(const char * prefix,
     case 0:
 	break;
     }
-#ifdef	SQLITE_HACK
+#ifdef	SQLITE_HACK_XXX
     if (rc || _olddbapi == _newdbapi)
 	return rc;
 
@@ -3770,7 +3763,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
     dbpath = rootdbpath = rpmGetPath(prefix, tfn, NULL);
     if (!(prefix[0] == '/' && prefix[1] == '\0'))
-	dbpath += strlen(prefix);
+	dbpath += strlen(prefix) - 1;
     tfn = _free(tfn);
 
     /*@-nullpass@*/
@@ -3793,7 +3786,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
     newdbpath = newrootdbpath = rpmGetPath(prefix, tfn, NULL);
     if (!(prefix[0] == '/' && prefix[1] == '\0'))
-	newdbpath += strlen(prefix);
+	newdbpath += strlen(prefix) - 1;
     tfn = _free(tfn);
 
     rpmMessage(RPMMESS_DEBUG, _("rebuilding database %s into %s\n"),
