@@ -1234,9 +1234,11 @@ expandMacro(MacroBuf mb)
 	/* Copy text until next macro */
 	switch(c) {
 	case '%':
-		if (*s != '%')
+		if (*s) {	/* Ensure not end-of-string. */
+		    if (*s != '%')
 			/*@switchbreak@*/ break;
-		s++;	/* skip first % in %% */
+		    s++;	/* skip first % in %% */
+		}
 		/*@fallthrough@*/
 	default:
 		SAVECHAR(mb, c);
@@ -1996,8 +1998,10 @@ rpmInitMacros(MacroContext mc, const char * macrofiles)
 	    continue;
 
 	/* Read macros from each file. */
-	for (i = 0; i < ac; i++)
+	for (i = 0; i < ac; i++) {
 	    (void) rpmLoadMacroFile(mc, av[i]);
+	    av[i] = _free(av[i]);
+	}
 	av = _free(av);
     }
     mfiles = _free(mfiles);
