@@ -9,6 +9,10 @@
 # include <machine/types.h>
 #endif
 
+#if HAVE_SYS_SOCKET_H
+# include <sys/socket.h>
+#endif
+
 #include <netinet/in.h>
 #include <arpa/inet.h>		/* XXX for inet_aton and HP-UX */
 
@@ -371,7 +375,11 @@ static ssize_t fdRead(void * cookie, /*@out@*/ char * buf, size_t count)
 /*@-boundswrite@*/
     /* HACK: flimsy wiring for davRead */
     if (fd->req != NULL) {
+#ifdef WITH_NEON
 	rc = davRead(fd, buf, (count > fd->bytesRemain ? fd->bytesRemain : count));
+#else
+	rc = -1;
+#endif
 	/* XXX Chunked davRead EOF. */
 	if (rc == 0)
 	    fd->bytesRemain = 0;
@@ -404,9 +412,13 @@ static ssize_t fdWrite(void * cookie, const char * buf, size_t count)
     fdstat_enter(fd, FDSTAT_WRITE);
 /*@-boundsread@*/
     /* HACK: flimsy wiring for davWrite */
-    if (fd->req != NULL)
+    if (fd->req != NULL) {
+#ifdef WITH_NEON
 	rc = davWrite(fd, buf, (count > fd->bytesRemain ? fd->bytesRemain : count));
-    else
+#else
+	return -1;
+#endif
+    } else
 	rc = write(fdno, buf, (count > fd->bytesRemain ? fd->bytesRemain : count));
 /*@=boundsread@*/
     fdstat_exit(fd, FDSTAT_WRITE, rc);
@@ -455,9 +467,13 @@ static int fdClose( /*@only@*/ void * cookie)
     fdstat_enter(fd, FDSTAT_CLOSE);
     /* HACK: flimsy wiring for davClose */
 /*@-branchstate@*/
-    if (fd->req != NULL)
+    if (fd->req != NULL) {
+#ifdef WITH_NEON
 	rc = davClose(fd);
-    else
+#else
+	return -1;
+#endif
+    } else
 	rc = ((fdno >= 0) ? close(fdno) : -2);
 /*@=branchstate@*/
     fdstat_exit(fd, FDSTAT_CLOSE, rc);
@@ -750,6 +766,7 @@ const char *urlStrerror(const char *url)
     return retstr;
 }
 
+#if !defined(HAVE_GETADDRINFO)
 #if !defined(USE_ALT_DNS) || !USE_ALT_DNS 
 static int mygethostbyname(const char * host,
 		/*@out@*/ struct in_addr * address)
@@ -800,14 +817,50 @@ static int getHostAddress(const char * host, /*@out@*/ struct in_addr * address)
 }
 /*@=compdef@*/
 /*@=boundsread@*/
+#endif
 
 static int tcpConnect(FD_t ctrl, const char * host, int port)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies ctrl, fileSystem, internalState @*/
 {
-    struct sockaddr_in sin;
     int fdno = -1;
     int rc;
+#ifdef	HAVE_GETADDRINFO
+    struct addrinfo hints, *res, *res0;
+    char pbuf[NI_MAXSERV];
+
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    sprintf(pbuf, "%d", port);
+    pbuf[sizeof(pbuf)-1] = '\0';
+    rc = FTPERR_FAILED_CONNECT;
+    if (getaddrinfo(host, pbuf, &hints, &res0) == 0) {
+	for (res = res0; res != NULL; res= res->ai_next) {
+	    if ((fdno = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) < 0)
+		continue;
+	    if (connect(fdno, res->ai_addr, res->ai_addrlen) < 0) {
+		close(fdno);
+		continue;
+	    }
+	    /* success */
+	    rc = 0;
+	    if (_ftp_debug) {
+		char hbuf[NI_MAXHOST];
+		getnameinfo(res->ai_addr, res->ai_addrlen, hbuf, sizeof(hbuf),
+				NULL, 0, NI_NUMERICHOST);
+		fprintf(stderr,"++ connect [%s]:%d on fdno %d\n",
+				/*@-unrecog@*/ hbuf /*@=unrecog@*/, port, fdno);
+	    }
+	    break;
+	}
+	freeaddrinfo(res0);
+    }
+    if (rc < 0)
+	goto errxit;
+
+#else	/* HAVE_GETADDRINFO */			    
+    struct sockaddr_in sin;
 
 /*@-boundswrite@*/
     memset(&sin, 0, sizeof(sin));
@@ -842,6 +895,7 @@ fprintf(stderr,"++ connect %s:%d on fdno %d\n",
 inet_ntoa(sin.sin_addr)
 /*@=unrecog =moduncon =evalorderuncon @*/ ,
 (int)ntohs(sin.sin_port), fdno);
+#endif	/* HAVE_GETADDRINFO */
 
     fdSetFdno(ctrl, (fdno >= 0 ? fdno : -1));
     return 0;
@@ -1174,12 +1228,17 @@ errxit2:
 int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 {
     urlinfo u = data->url;
+#if !defined(HAVE_GETADDRINFO)
     struct sockaddr_in dataAddress;
+#endif	/* HAVE_GETADDRINFO */
+    char remoteIP[NI_MAXHOST];
     char * cmd;
     int cmdlen;
     char * passReply;
     char * chptr;
     int rc;
+    int epsv;
+    int port;
 
 /*@-boundswrite@*/
     URLSANE(u);
@@ -1214,8 +1273,35 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 	data->contentLength = cl;
     }
 
+    epsv = 0;
     passReply = NULL;
-    rc = ftpCommand(u, &passReply, "PASV", NULL);
+#ifdef HAVE_GETNAMEINFO
+    rc = ftpCommand(u, &passReply, "EPSV", NULL);
+    if (rc==0) {
+#ifdef HAVE_GETADDRINFO
+	struct sockaddr_storage ss;
+#else /* HAVE_GETADDRINFO */
+	struct sockaddr_in ss;
+#endif /* HAVE_GETADDRINFO */
+	int size;
+	/* we need to know IP of remote host */
+	size=sizeof(ss);
+	if ((getpeername(fdFileno(c2f(u->ctrl)), (struct sockaddr *)&ss, &size) == 0) &&
+			(getnameinfo((struct sockaddr *)&ss, size, remoteIP, sizeof(remoteIP),
+				NULL, 0, NI_NUMERICHOST) == 0))
+		epsv++;
+	else {
+		/* abort EPSV and fall back to PASV */
+		rc = ftpCommand(u, &passReply, "ABOR", NULL);
+		if (rc) {
+		    rc = FTPERR_PASSIVE_ERROR;
+		    goto errxit;
+		}
+	}
+    }
+    if (epsv==0)
+#endif /* HAVE_GETNAMEINFO */
+        rc = ftpCommand(u, &passReply, "PASV", NULL);
     if (rc) {
 	rc = FTPERR_PASSIVE_ERROR;
 	goto errxit;
@@ -1230,6 +1316,15 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
     if (*chptr != ')') return FTPERR_PASSIVE_ERROR;
     *chptr-- = '\0';
 
+    if (epsv) {
+	int i;
+        if(sscanf(passReply,"%*c%*c%*c%d%*c",&i) != 1) {
+	   rc = FTPERR_PASSIVE_ERROR;
+	   goto errxit;
+	}
+	port = i;
+    } else {
+ 
     while (*chptr && *chptr != ',') chptr--;
     if (*chptr != ',') return FTPERR_PASSIVE_ERROR;
     chptr--;
@@ -1241,13 +1336,11 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
        port number portion */
 
     {	int i, j;
-	memset(&dataAddress, 0, sizeof(dataAddress));
-	dataAddress.sin_family = AF_INET;
 	if (sscanf(chptr, "%d,%d", &i, &j) != 2) {
 	    rc = FTPERR_PASSIVE_ERROR;
 	    goto errxit;
 	}
-	dataAddress.sin_port = htons((((unsigned)i) << 8) + j);
+	port = (((unsigned)i) << 8) + j;
     }
 
     chptr = passReply;
@@ -1255,9 +1348,75 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 	if (*chptr == ',') *chptr = '.';
     }
 /*@=boundswrite@*/
+    sprintf(remoteIP, "%s", passReply);
+    } /* if (epsv) */
+
+#ifdef HAVE_GETADDRINFO
+    {
+        struct addrinfo hints, *res, *res0;
+	char pbuf[NI_MAXSERV];
+
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_family = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_flags = AI_NUMERICHOST;
+	sprintf(pbuf, "%d", port);
+	pbuf[sizeof(pbuf)-1] = '\0';
+	if (getaddrinfo(remoteIP, pbuf, &hints, &res0)) {
+	    rc = FTPERR_PASSIVE_ERROR;
+	    goto errxit;
+	}
+
+	for (res = res0; res != NULL; res = res->ai_next) {
+	    rc = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+	    fdSetFdno(data, (rc >= 0 ? rc : -1));
+	    if (rc < 0) {
+	        if (res->ai_next)
+		    continue;
+		else {
+		    rc = FTPERR_FAILED_CONNECT;
+		    freeaddrinfo(res0);
+		    goto errxit;
+		}
+	    }
+	    data = fdLink(data, "open data (ftpReq)");
+
+	    /* XXX setsockopt SO_LINGER */
+	    /* XXX setsockopt SO_KEEPALIVE */
+	    /* XXX setsockopt SO_TOS IPTOS_THROUGHPUT */
+	    
+	    {
+		int criterr = 0;
+	        while (connect(fdFileno(data), res->ai_addr, res->ai_addrlen) < 0) {
+	            if (errno == EINTR)
+		        continue;
+		    criterr++;
+		}
+		if (criterr) {
+		    if (res->ai_addr) {
+		        fdClose(data);
+		        continue;
+		    } else {
+		        rc = FTPERR_PASSIVE_ERROR;
+		        freeaddrinfo(res0);
+		        goto errxit;
+		    }
+		}
+	    }
+	    /* success */
+	    rc = 0;
+	    break;
+	}
+	freeaddrinfo(res0);
+    }
+	    
+#else /* HAVE_GETADDRINFO */
+    memset(&dataAddress, 0, sizeof(dataAddress));
+    dataAddress.sin_family = AF_INET;
+    dataAddress.sin_port = htons(port);
 
     /*@-moduncon@*/
-    if (!inet_aton(passReply, &dataAddress.sin_addr)) {
+    if (!inet_aton(remoteIP, &dataAddress.sin_addr)) {
 	rc = FTPERR_PASSIVE_ERROR;
 	goto errxit;
     }
@@ -1285,6 +1444,7 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 	goto errxit;
     }
     /*@=internalglobs@*/
+#endif /* HAVE_GETADDRINFO */
 
 if (_ftp_debug)
 fprintf(stderr, "-> %s", cmd);
@@ -1598,6 +1758,7 @@ static int httpReq(FD_t ctrl, const char * httpCmd, const char * httpArg)
     urlinfo u;
     const char * host;
     const char * path;
+    char hthost[NI_MAXHOST];
     int port;
     int rc;
     char * req;
@@ -1610,6 +1771,10 @@ assert(ctrl != NULL);
 
     if (((host = (u->proxyh ? u->proxyh : u->host)) == NULL))
 	return FTPERR_BAD_HOSTNAME;
+    if (strchr(host, ':'))
+	sprintf(hthost, "[%s]", host);
+    else
+	strcpy(hthost, host);
 
     if ((port = (u->proxyp > 0 ? u->proxyp : u->port)) < 0) port = 80;
     path = (u->proxyh || u->proxyp > 0) ? u->url : httpArg;
@@ -1639,7 +1804,7 @@ Host: y:z\r\n\
 Accept: text/plain\r\n\
 Transfer-Encoding: chunked\r\n\
 \r\n\
-") + strlen(httpCmd) + strlen(path) + sizeof(VERSION) + strlen(host) + 20;
+") + strlen(httpCmd) + strlen(path) + sizeof(VERSION) + strlen(hthost) + 20;
 
 /*@-boundswrite@*/
     req = alloca(len);
@@ -1653,7 +1818,7 @@ Host: %s:%d\r\n\
 Accept: text/plain\r\n\
 Transfer-Encoding: chunked\r\n\
 \r\n\
-",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, host, port);
+",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, hthost, port);
 } else {
     sprintf(req, "\
 %s %s HTTP/1.%d\r\n\
@@ -1661,7 +1826,7 @@ User-Agent: rpm/%s\r\n\
 Host: %s:%d\r\n\
 Accept: text/plain\r\n\
 \r\n\
-",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, host, port);
+",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, hthost, port);
 }
 /*@=boundswrite@*/
 
@@ -2029,6 +2194,56 @@ exit:
 }
 /*@=nullstate@*/
 
+#ifndef WITH_NEON
+/*@-nullstate@*/        /* FIX: u->{ctrl,data}->url undef after XurlLink. */
+static /*@null@*/ FD_t httpOpen(const char * url, /*@unused@*/ int flags,
+                /*@unused@*/ mode_t mode, /*@out@*/ urlinfo * uret)
+        /*@globals internalState @*/
+        /*@modifies *uret, internalState @*/
+{
+    urlinfo u = NULL;
+    FD_t fd = NULL;
+
+#if 0   /* XXX makeTempFile() heartburn */
+    assert(!(flags & O_RDWR));
+#endif
+    if (urlSplit(url, &u))
+        goto exit;
+
+    if (u->ctrl == NULL)
+        u->ctrl = fdNew("persist ctrl (httpOpen)");
+    if (u->ctrl->nrefs > 2 && u->data == NULL)
+        u->data = fdNew("persist data (httpOpen)");
+
+    if (u->ctrl->url == NULL)
+        fd = fdLink(u->ctrl, "grab ctrl (httpOpen persist ctrl)");
+    else if (u->data->url == NULL)
+        fd = fdLink(u->data, "grab ctrl (httpOpen persist data)");
+    else
+        fd = fdNew("grab ctrl (httpOpen)");
+
+    if (fd) {
+        fdSetIo(fd, ufdio);
+        fd->ftpFileDoneNeeded = 0;
+        fd->rd_timeoutsecs = httpTimeoutSecs;
+        fd->contentLength = fd->bytesRemain = -1;
+        fd->url = urlLink(u, "url (httpOpen)");
+        fd = fdLink(fd, "grab data (httpOpen)");
+        fd->urlType = URL_IS_HTTP;
+    }
+
+exit:
+/*@-boundswrite@*/
+    if (uret)
+        *uret = u;
+/*@=boundswrite@*/
+    /*@-refcounttrans@*/
+    return fd;
+    /*@=refcounttrans@*/
+}
+/*@=nullstate@*/
+#endif
+
 static /*@null@*/ FD_t ufdOpen(const char * url, int flags, mode_t mode)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
@@ -2067,7 +2282,11 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
     case URL_IS_HKP:
+#ifdef WITH_NEON
 	fd = davOpen(url, flags, mode, &u);
+#else
+	fd = httpOpen(url, flags, mode, &u);
+#endif
 	if (fd == NULL || u == NULL)
 	    break;
 
@@ -2075,7 +2294,11 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
 		?  ((flags & O_APPEND) ? "PUT" :
 		   ((flags & O_CREAT) ? "PUT" : "PUT"))
 		: "GET");
+#ifdef WITH_NEON
 	u->openError = davReq(fd, cmd, path);
+#else
+	u->openError = httpReq(fd, cmd, path);
+#endif
 	if (u->openError < 0) {
 	    /* XXX make sure that we can exit through ufdClose */
 	    fd = fdLink(fd, "error ctrl (ufdOpen HTTP)");
@@ -3094,6 +3317,7 @@ int Fileno(FD_t fd)
 {
     int i, rc = -1;
 
+    if (fd == NULL) return -1;
     if (fd->req != NULL)
 	rc = 123456789;	/* HACK: https has no steenkin fileno. */
     else
